/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("indexing");

/**
	@class Singleton that provides functions to validate tokens. <br />
*/
BookmarkTools.indexing.Token = {
	/**
		Returns true if the token is an url, otherwise false.
		@param {string} token
		@return true or false
	*/
	isURL : function(token){
		var regexp = BookmarkTools.indexing.PATTERN_TOK_URL;
		return regexp.test(token);
	},
	/**
		Returns true if the token is an ip adress, otherwise false.
		@param {string} token
		@return true or false
	*/
	isIPAdress : function(token){
		var regObj = new RegExp(BookmarkTools.indexing.PATTERN_TOK_IP_ADRESS);
		return regObj.test(token);
	},
	/**
		Returns true if the token is an email adress, otherwise false.
		@param {string} token
		@return true or false
	*/
	isEMailAdress : function(token){
		var regexp = BookmarkTools.indexing.PATTERN_TOK_EMAIL;
		return regexp.test(token); 
	}
};

/**
	@class Basic tokenizer class containing common parse functions needed for every token stream. <br />
*/
BookmarkTools.indexing.Tokenizer = function(){
	/**
		Replaces whitespaces like /\f, /\n, /\t, /\v with an single space.
		@param {string} text
		@return whitespace filtered content
	*/
	this.replaceWhiteSpaces = function(text){
		return text.replace(
			BookmarkTools.indexing.PATTERN_WHITESPACES, 
			BookmarkTools.utils.StringUtils.SPACE
		);
	};
	/**
		Removes numbers from the given content.
		@param {string} text
		@return number filtered content
	*/
	this.removeNumbers = function(text){
		return text.replace(
			BookmarkTools.indexing.PATTERN_NUMBERS, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	};
	/**
		Removes html color expressions like #ff0000.
		@param {string} text
		@return filtered content
	*/
	this.replaceColorExpression = function(text){
		return text.replace(
			BookmarkTools.indexing.PATTERN_CSS_COLOR, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	};
	/**
		Removes css unit expressions like 12px or 1.5em.
		@param {string} text
		@return filtered content
	*/
	this.replaceCSSUnitExpression = function(text){
		return text.replace(
			BookmarkTools.indexing.PATTERN_CSS_UNIT, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	};
	
};

/**
	@class Simple whitespace tokenizer that uses space as delimiter and keeps ip and email adresses and urls. <br />
*/
BookmarkTools.indexing.WhitespaceTokenizer = function(){};
BookmarkTools.indexing.WhitespaceTokenizer.prototype = new BookmarkTools.indexing.Tokenizer();

/**
	Removes punctuation and useless characters like brackets or slashes.
	@param {string} text
	@return filtered content
*/
BookmarkTools.indexing.WhitespaceTokenizer.prototype.replaceMarks = function(text){
	return text.replace(
		BookmarkTools.indexing.PATTERN_MARKS, 
		BookmarkTools.utils.StringUtils.SPACE
	);
};
/**
	Returns an filtered token stream. Used to tokenize well-formed documents.
	@param {string} text
	@return token stream
*/
BookmarkTools.indexing.WhitespaceTokenizer.prototype.tokenize = function(text){
	text = this.replaceWhiteSpaces(text);
	text = text.toLowerCase();
	
	var tokens = text.split(BookmarkTools.utils.StringUtils.SPACE);
	
	var tokensList = [], token, trimmedToken, i, index = 0;
	
	for(i = 0, l = tokens.length ;i < l; i++){
		token = tokens[i];
		if(BookmarkTools.indexing.Token.isURL(token) || BookmarkTools.indexing.Token.isIPAdress(token) ||
			BookmarkTools.indexing.Token.isEMailAdress(token)){
			tokensList[index++] = token;
		}else {
			token = this.replaceMarks(token);
			trimmedToken = BookmarkTools.utils.StringUtils.trim(token);
			
			if(BookmarkTools.utils.StringUtils.isNotEmpty(trimmedToken))
				tokensList[index++] = trimmedToken;
		}
	}
	return tokensList;
};

/**
	@class Lazy tokenizer that uses space as delimiter and tries to remove html and css specific expressions. <br />
*/
BookmarkTools.indexing.LazyTokenizer = function(){};
BookmarkTools.indexing.LazyTokenizer.prototype = new BookmarkTools.indexing.Tokenizer();

/**
	Removes punctuation and useless characters like brackets or slashes.
	@param {string} text
	@return filtered content
*/
BookmarkTools.indexing.LazyTokenizer.prototype.replaceMarks = function(text){
	return text.replace(
		BookmarkTools.indexing.PATTERN_MARKS_LAZY, 
		BookmarkTools.utils.StringUtils.SPACE
	);
};
/**
	Returns an filtered token stream. Used to tokenize documents which aren't well-formed.
	@param {string} text
	@return token stream
*/
BookmarkTools.indexing.LazyTokenizer.prototype.tokenize = function(text){
	var preparedText = this.replaceColorExpression(text);
	preparedText = this.replaceCSSUnitExpression(preparedText);
	
	preparedText = this.replaceWhiteSpaces(preparedText);
	preparedText = preparedText.toLowerCase();
	
	var tokens = preparedText.split(BookmarkTools.utils.StringUtils.SPACE);
	var tokensList = [], token, trimmedToken, i, index = 0;
	
	for(i = 0, l = tokens.length ;i < l; i++){
		token = tokens[i];
		if(BookmarkTools.indexing.Token.isURL(token) || BookmarkTools.indexing.Token.isIPAdress(token) ||
			BookmarkTools.indexing.Token.isEMailAdress(token)){
			tokensList[index++] = token;
		}else {
			token = this.replaceMarks(token);
			trimmedToken = BookmarkTools.utils.StringUtils.trim(token);
			
			if(BookmarkTools.utils.StringUtils.isNotEmpty(trimmedToken))
				tokensList[index++] = trimmedToken;
		}
	}
	return tokensList;
};
